#include "Versions.h"

#include <Wire.h>
#include "rgb_lcd.h"
#include "DHT.h"
#include "TSDigitalTube.h"
#include "TSTemperature.h"
#include "TSServoOnline.h"
#include "TSRGBLed.h"
#include "TSLedBarOnline.h"
#include "Ultrasonic.h"
#include "TSRobunoOnline.h"
#include "TSi2cMotorDriver.h"
#include "TSSerial.h"
#include "TSTools.h"
#include "TSLedStrip.h"
#include "RCSwitch.h"
#include "virtualGroveConnector.h"
#include "Seeed_RFID.h"
#include "Seeed_GasSensor.h"
#include "Seeed_DustSensor.h"
#include "Seeed_ColorSensor.h"
#include "Adafruit_TCS34725.h"
#include "Seeed_BMP280.h"
#include "TSPlayer.h"
#include "ADXL345.h"
#include "Adafruit_Sensor.h"
#include "Adafruit_HMC5883_U.h"
#include "Seeed_FingerHeartRate.h"
#include "Adafruit_Fingerprint.h"

typedef enum
{
  I2C_COMPASS = 0x00,
  I2C_COLORSENSOR,
  I2C_COLORSENSORV2,
  I2C_FINGERHEARTRATE,
  I2C_FINGERPRINT,
  I2C_BMP280,
  I2C_ACCELEROMETER,
  I2C_MAX
} i2c_t;

virtualGroveConnector *digitalConnector[8] =  {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
virtualGroveConnector *i2CConnector[I2C_MAX] ;

rgb_lcd 			    rgbLcd;
TSDigitalTube     digitalTube;
TSServo 			    servo;
TSRGBLed 			    rgbLed;
TSLedBar 			    ledBar;
TSi2cMotorDriver 	i2cMotorDriver;
TSRobuno          robuno;
TSSerial          bt;

typedef struct MeModule
{
  int device;
  int port;
  int slot;
  int pin;
  int index;
  float values[3];
} MeModule;

union {
  byte byteVal[4];
  float floatVal;
  long longVal;
} val;

union {
  byte byteVal[8];
  double doubleVal;
} valDouble;

union {
  byte byteVal[2];
  short shortVal;
} valShort;

MeModule modules[12];
#if defined(__AVR_ATmega32U4__)
int analogs[12] = {A0, A1, A2, A3, A4, A5, A6, A7, A8, A9, A10, A11};
#else
int analogs[8] = {A0, A1, A2, A3, A4, A5, A6, A7};
#endif
String mVersion = "1.0.005";
boolean isAvailable = false;
boolean isBluetooth = false;

int len = 52;
char buffer[52];
char bufferBt[52];
byte index = 0;
byte dataLen;
byte modulesLen = 0;
boolean isStart = false;
unsigned char irRead;
char serialRead;
#define VERSION 0
#define ULTRASONIC_SENSOR   1
#define TEMPERATURE_SENSOR  2 //GROV49 v1.2
#define LIGHT_SENSOR        3
#define POTENTIONMETER      4
#define JOYSTICK            5
#define GYRO                6
#define SOUND_SENSOR        7
#define RGBLED              8
#define SEVSEG              9
#define MOTOR               10
#define SERVO               11
#define ENCODER             12
#define IR                  13
#define PIRMOTION           15
#define INFRARED            16
#define LINEFOLLOWER        17
#define SHUTTER             20
#define LIMITSWITCH         21
#define BUTTON              22
#define DIGITAL             30
#define ANALOG              31
#define PWM                 32
#define SERVO_PIN           33
#define TONE                34

//TS
#define I2CDISPLAY            42
#define DHT11_TEMPERATURE     43  //DHT 11
#define DHT11_HUMIDITY        44  //DHT 11
#define SERVO_ANGLE           45
#define SERVO_CONT            46
#define HEART_RATE            47
#define LEDBAR                48
#define I2CDISPLAY_CLEAR      49
#define TIMER                 50
#define ROBUNO_RUN            51
#define ROBUNO_STOP           52
#define ROBUNO_READ           53
#define ROBUNO_WRITE          54
#define ROBUNO_LDR            55
#define ROBUNO_SERVO          56
#define I2CDISPLAY_BACKLIGHT  57
#define APPINVENTOR_SEND      58
#define APPINVENTOR_RECEIVE   59
#define APPINVENTOR_AVAILABLE 60
#define DHT22_TEMPERATURE     61
#define DHT22_HUMIDITY        62
#define BMP280_TEMPERATURE    63
#define BMP280_PRESSURE       64
#define HUMIDITY              65  //GROV24
#define RF433_AVAILABLE       66  //GROV53
#define RF433_RECEIVE         67
#define RF433_SEND            68
#define LED_STRIP_PIXEL       69
#define LED_STRIP_BRIGHTNESS  70
#define LED_STRIP_PATTERN     71
#define MP3_PLAYER            72  //GROV111
#define ACCELEROMETER         73

#define SENSOR_RFID_AVAIL       80
#define SENSOR_RFID_READ        81
#define SENSOR_HEARTRATE_AVAIL  82
#define SENSOR_HEARTRATE_READ   83
#define SENSOR_CO2_AVAIL        84
#define SENSOR_CO2_READ         85 //CO2, temp
#define SENSOR_O2_READ          86
#define SENSOR_DUST_READ        87

#define SENSOR_COLORRGB     88 //r,g,b
#define SENSOR_COLORRGBV2   100
#define SENSOR_COMPASS3D    89 //x,y,z

#define SENSOR_FINGERPRINT_DELETE        90
#define SENSOR_FINGERPRINT_ENROLLPASS1   91
#define SENSOR_FINGERPRINT_ENROLLPASS2   92
#define SENSOR_FINGERPRINT_READAVAILABLE 93
#define SENSOR_FINGERPRINT_READID        94
//


#define GET 1
#define RUN 2
#define RESET 4
#define START 5

float angleServo 		= 90.0;
unsigned char prevc 	= 0;
double lastTime 		= 0.0;
double currentTime 		= 0.0;

void setup()
{
  for (int ii2c = 0; ii2c < I2C_MAX; ii2c++)
  {
    i2CConnector[ii2c] = NULL;
  }
  robuno.stop();
  pinMode(13, OUTPUT);
  digitalWrite(13, HIGH);
  delay(300);
  digitalWrite(13, LOW);


  Serial.begin(115200);
  sendString(mVersion);
#if defined(__AVR_ATmega32U4__)
  Serial1.begin(115200);
#endif

  Wire.begin();
  rgbLcd.begin(16, 2);
}


void loop() {
  currentTime = millis() / 1000.0 - lastTime;
  /*if(ir.buttonState()==1){
    if(ir.available()>0){
    irRead = ir.read();
    }
    }else{
    irRead = 0;
    }*/
  //Recepteur IR
  /*unsigned char t = ir.poll();
    if (t != 0)
    {
    Serial.println(t);
    }
  */

  if (readSerial())
  {
    unsigned char c = serialRead & 0xff;

    if (c == 0x55 && isStart == false)
    {
      if (prevc == 0xff)
      {
        index = 1;
        isStart = true;
      }
    } else {
      prevc = c;
      if (isStart)
      {
        if (index == 2)
        {
          dataLen = c;
        } else if (index > 2) {
          dataLen--;
        }
        writeBuffer(index, c);
      }
    }
    index++;
    if (index > 51)
    {
      index = 0;
      isStart = false;
    }
    if (isStart && dataLen == 0 && index > 3)
    {
      isStart = false;
      parseData();
      index = 0;
    }
  }
}

unsigned char readBuffer(int index) {
  return isBluetooth ? bufferBt[index] : buffer[index];
}

void writeBuffer(int index, unsigned char c) {
  if (isBluetooth) {
    bufferBt[index] = c;
  } else {
    buffer[index] = c;
  }
}

void writeHead() {
  writeSerial(0xf1);
  writeSerial(0xff);
  writeSerial(0x55);
}

void writeEnd() {
  Serial.println();
#if defined(__AVR_ATmega32U4__)
  Serial1.println();
#endif
}

void writeSerial(unsigned char c) {
  Serial.write(c);
#if defined(__AVR_ATmega32U4__)
  Serial1.write(c);
#endif
}

bool readSerial()
{
  if (Serial.available())
  {
    serialRead = Serial.read();
    return true;
  }
  return false;
}
/*****************************************************
	parseData
		ff 55 len idx action device port  slot  data a
		0  1  2   3   4      5      6     7     8
****************************************************/
void parseData() {
  isStart		 = false;
  int idx 	= readBuffer(3);
  int action 	= readBuffer(4);
  int device 	= readBuffer(5);
  switch (action) {
    case GET:
      {
        writeHead();
        writeSerial(idx);
        readSensor(device);
        writeEnd();
        break;
      }
    case RUN:
      {
        runModule(device);
        callOK();
        break;
      }
    case RESET:
      {
        callOK();
        break;
      }
    case START:
      {
        callOK();
        break;
      }
  }
}

void callOK() {
  writeSerial(0xf0);
  writeSerial(0xff);
  writeSerial(0x55);
  writeEnd();
}


//--------------------------------
void sendByte(char c) {
  writeSerial(1);
  writeSerial(c);
}
//--------------------------------
void sendString(String s) {
  int l = s.length();
  writeSerial(4);
  writeSerial(l);
  for (int i = 0; i < l; i++) {
    writeSerial(s.charAt(i));
  }
}
//--------------------------------
void sendFloat(float value) {
  writeSerial(0x2);
  val.floatVal = value;
  writeSerial(val.byteVal[0]);
  writeSerial(val.byteVal[1]);
  writeSerial(val.byteVal[2]);
  writeSerial(val.byteVal[3]);
}
//--------------------------------
void sendShort(double value) {
  writeSerial(3);
  valShort.shortVal = value;
  writeSerial(valShort.byteVal[0]);
  writeSerial(valShort.byteVal[1]);
  writeSerial(valShort.byteVal[2]);
  writeSerial(valShort.byteVal[3]);
}

void sendShort(unsigned long value)
{
  writeSerial(6);
  val.longVal = value;
  writeSerial(val.byteVal[0]);
  writeSerial(val.byteVal[1]);
  writeSerial(val.byteVal[2]);
  writeSerial(val.byteVal[3]);
}
//--------------------------------
void sendDouble(double value) {
  writeSerial(2);
  valDouble.doubleVal = value;
  writeSerial(valDouble.byteVal[0]);
  writeSerial(valDouble.byteVal[1]);
  writeSerial(valDouble.byteVal[2]);
  writeSerial(valDouble.byteVal[3]);
}
//--------------------------------
short readShort(int idx) {
  valShort.byteVal[0] = readBuffer(idx);
  valShort.byteVal[1] = readBuffer(idx + 1);
  return valShort.shortVal;
}
//--------------------------------
float readFloat(int idx) {
  val.byteVal[0] = readBuffer(idx);
  val.byteVal[1] = readBuffer(idx + 1);
  val.byteVal[2] = readBuffer(idx + 2);
  val.byteVal[3] = readBuffer(idx + 3);
  return val.floatVal;
}


//--------------------------------
void runModule(int device)
{
  /**************************************************
      ff 55    len  idx  action  device  port  param1 param2 ...
  	0  1  	   2   3     4      5      6      7     8
  ***************************************************/
  //0xff 0x55  0x6  0x0  0x1     0xa     0x9   0x0   0x0   0xa
  int port = readBuffer(6);
  int pin  = port;
  switch (device) {
    case MOTOR:
      {
        int direction 	= readBuffer(7);
        int speed1 		= readBuffer(8);
        int speed2 		= readBuffer(9);
        i2cMotorDriver.MotorSpeedSetAB(speed1, speed2);
        i2cMotorDriver.MotorDirectionSet(direction);
        break;
      }
    case RGBLED:
      {
        int id 		= readBuffer(7);
        int red 	= readBuffer(8);
        int green 	= readBuffer(9);
        int blue 	= readBuffer(10);
        rgbLed.setColorRGB(pin, pin + 1, id, red, green, blue);
        break;
      }
    case SERVO:
    case SEVSEG:
      {
        digitalTube.show(pin, readBuffer(7), readBuffer(8), (bool)readBuffer(9));
        break;
      }
    case LIGHT_SENSOR:
    case SHUTTER:
    case SERVO_PIN:
      break;
    case PWM:
      {
        pinMode(pin, OUTPUT);
        int v = readBuffer(7);
        analogWrite(pin, v);
        break;
      }
    case TONE:
      {
        pinMode(pin, OUTPUT);
        int hz = readShort(7);
        int ms = readShort(9);
        if (ms > 0) {
          tone(pin, hz, ms);
        } else {
          noTone(pin);
        }
        break;
      }
    case TIMER:
      lastTime = millis() / 1000.0;
      break;

    case DIGITAL:
      {
        pinMode(pin, OUTPUT);
        int v = readBuffer(7);
        digitalWrite(pin, v);
        break;
      }

    //************************************//
    // 			TS actuators
    //************************************//
    case SERVO_ANGLE:
      {
        int val 	= readBuffer(7);
        int speed 	= readBuffer(8);
        int trim 	= readBuffer(9);
        servo.write(pin, val, speed);
        break;
      }
    case SERVO_CONT:
      {
        int val 	= readBuffer(7);
        bool dir 	= readBuffer(8);
        int trim 	= readBuffer(9);
        servo.write(pin, val, dir, trim - 90);
        break;
      }
    case I2CDISPLAY:
      {
        int line = readBuffer(7);
        int len  = readBuffer(8);
        String s;
        for (byte i = 9; i < 9 + len; i++)
        {
          s += buffer[i];
        }
        rgbLcd.setCursor(0, line);
        rgbLcd.print(s);
        rgbLcd.print("                ");
        break;
      }
    case I2CDISPLAY_CLEAR:
      {
        rgbLcd.clear();
        break;
      }
    case I2CDISPLAY_BACKLIGHT:
      {
        int r = pin;
        int g = readBuffer(7);
        int b = readBuffer(8);
        rgbLcd.setRGB(r, g, b);
        break;
      }
    case LEDBAR:
      {
        int level = readBuffer(7);
        ledBar.setLevel(pin, (float)level);
        break;
      }

    case ROBUNO_RUN:
      {
        byte direction = pin;
        byte speed = readBuffer(7);
        robuno.run(direction, speed);
        break;
      }
    case ROBUNO_STOP:
      {
        robuno.stop();
        break;
      }
    case ROBUNO_SERVO:
      {
        robuno.enableServo();
        break;
      }
    case ROBUNO_WRITE:
      {
        bool state = readBuffer(7);
        robuno.write(pin, state);
        break;
      }

    case APPINVENTOR_SEND:
      {
        int len = readBuffer(7);
        String s;
        for (byte i = 8; i < 8 + len; i++)
        {
          s += buffer[i];
        }
        bt.send(pin, pin + 1, s);
        break;
      }
    case RF433_SEND:
      break;
#ifdef INCLUDE_LEDSTRIP
    case LED_STRIP_PIXEL:
      {
        byte leds = readBuffer(7);
        byte led = readBuffer(8);
        byte r = readBuffer(9);
        byte g = readBuffer(10);
        byte b = readBuffer(11);
        if (digitalConnector[pin] == NULL)
        {
          digitalConnector[pin] = new TSLedStrip;
          ((TSLedStrip*)digitalConnector[pin])->init(pin, leds);
        }
        ((TSLedStrip*)digitalConnector[pin])->mStrip->setPixelColor(led, r, g, b);
        ((TSLedStrip*)digitalConnector[pin])->mStrip->show();
        break;
      }
    case LED_STRIP_BRIGHTNESS:
      {
        byte leds = readBuffer(7);
        byte value = readBuffer(8);
        if (digitalConnector[pin] == NULL)
        {
          digitalConnector[pin] = new TSLedStrip;
          ((TSLedStrip*)digitalConnector[pin])->init(pin, leds);
        }
        ((TSLedStrip*)digitalConnector[pin])->mStrip->setBrightness(value);
        ((TSLedStrip*)digitalConnector[pin])->mStrip->show();
        break;
      }
    case LED_STRIP_PATTERN:
      {
        byte leds = readBuffer(7);
        byte pattern = readBuffer(8);
        byte r = readBuffer(9);
        byte g = readBuffer(10);
        byte b = readBuffer(11);
        if (digitalConnector[pin] == NULL)
        {
          digitalConnector[pin] = new TSLedStrip;
          ((TSLedStrip*)digitalConnector[pin])->init(pin, leds);
        }
        ((TSLedStrip*)digitalConnector[pin])->animation(pattern, r, g, b);
        break;
      }
#endif //LEDSTRIP
#ifdef INCLUDE_MP3PLAYER
    case MP3_PLAYER:
      if(digitalConnector[pin] == NULL)
      {
        digitalConnector[pin] = new TSPlayer;
        ((TSPlayer*)digitalConnector[pin])->init(pin);
      }
      ((TSPlayer*)digitalConnector[pin])->command(readBuffer(7));
      break;
#endif
  }
}

#define _PARAM_LENGTH 	7
#define _PARAM1_ 	_PARAM_LENGTH+1
#define _PARAM2_ 	_PARAM1_+1
#define _PARAM3_ 	_PARAM2_+1

void readSensor(int device) {
  /**************************************************
    ff 55 len idx action device port slot data a
    0  1  2   3   4      5      6    7    8
  ***************************************************/
  float value = 0.0;
  int port, slot, pin;
  port 	= readBuffer(6);
  pin 	= port;
  switch (device) {
    case  ULTRASONIC_SENSOR:
      {
        pinMode(pin, INPUT);
        Ultrasonic us(pin);
        sendShort((double)us.MeasureInCentimeters());
        break;
      }
    case  TEMPERATURE_SENSOR:
      {
        pin = analogs[pin];
        pinMode(pin, INPUT);
        sendFloat(getTemperature(analogRead(pin), 100000.0f, 4275));
        break;
      }
    case  LIGHT_SENSOR:
    case  SOUND_SENSOR:
    case  POTENTIONMETER:
    case  JOYSTICK:
      {
        pinMode(pin, INPUT);
        sendByte(analogRead(pin) > 1000);
        break;
      }
    case  INFRARED:
    case  PIRMOTION:
    case  LINEFOLLOWER:
    case  LIMITSWITCH:
    case  GYRO:
      break;
    case  VERSION:
      {
        sendString(mVersion);
        break;
      }

    case  DIGITAL:
      {
        pinMode(pin, INPUT);
        sendFloat(digitalRead(pin));
        break;
      }
    case  ANALOG:
      {
        pin = analogs[pin];
        pinMode(pin, INPUT);
        sendFloat(analogRead(pin));
        break;
      }
    case TIMER:
      sendFloat((float)currentTime);
      break;

    //************************************//
    // 			TS sensors
    //************************************//

    //--------------------
    case DHT11_TEMPERATURE:
      if (digitalConnector[pin] == NULL)
      {
        digitalConnector[pin] = new DHT(pin, 11);
      }
      sendFloat( ((DHT *)digitalConnector[pin])->readTemperature() );
      break;
    case DHT11_HUMIDITY:
      if (digitalConnector[pin] == NULL)
      {
        digitalConnector[pin] = new DHT(pin, 11);
      }
      sendFloat( ((DHT *)digitalConnector[pin])->readHumidity() );
      break;
    case DHT22_TEMPERATURE:
      if (digitalConnector[pin] == NULL)
      {
        digitalConnector[pin] = new DHT(pin, 22);
      }
      sendFloat( ((DHT *)digitalConnector[pin])->readTemperature() );
      break;
    case DHT22_HUMIDITY:
      if (digitalConnector[pin] == NULL)
      {
        digitalConnector[pin] = new DHT(pin, 22);
      }
      sendFloat( ((DHT *)digitalConnector[pin])->readHumidity() );
      break;
#ifdef INCLUDE_BMP280
    case BMP280_TEMPERATURE:
      {
        BMP280 *bmp280 = (BMP280*)i2CConnector[I2C_BMP280];
        if (bmp280 == NULL)
        {
          bmp280 = new BMP280;
          i2CConnector[I2C_BMP280] = bmp280;
          bmp280->init();
        }
        sendFloat(bmp280->getTemperature());
      }
      break;
    case BMP280_PRESSURE:
      {
        BMP280 *bmp280 = (BMP280*)i2CConnector[I2C_BMP280];
        if (bmp280 == NULL)
        {
          bmp280 = new BMP280;
          i2CConnector[I2C_BMP280] = bmp280;
          bmp280->init();
        }
        float pressure = (float)bmp280->getPressure();
        sendFloat(pressure);
      }
      break;
#endif
    case HUMIDITY:
      sendByte(getHumidity(pin));
      break;
    //--------------------
    // RFID Sensor
    //--------------------
    case SENSOR_RFID_AVAIL:
      {
        if (digitalConnector[pin] == NULL)  										// test si le port à déjà été déclaré
          digitalConnector[pin] = new Seeed_RFID(pin, pin + 1);					// aucun capteur sur le port, je lui affecte un capteur type RFID (fait une seule fois)
        sendByte( ((Seeed_RFID *)digitalConnector[pin])->isAvailable() );		// retourne la valeur si capteur est présent?
        break;
      }
    case SENSOR_RFID_READ:
      {
        if (digitalConnector[pin] == NULL)  										// test si le port à déjà été déclaré
          digitalConnector[pin] = new Seeed_RFID(pin, pin + 1);					// aucun capteur sur le port, je lui affecte un capteur type RFID(fait une seule fois)
        sendString(String( ((Seeed_RFID *)digitalConnector[pin])->cardNumber() )); // retourne la valeur du capteur lu
        break;
      }
          case SENSOR_COLORRGBV2:
      {
        // Bug must be redeclared every time (?)
        if (i2CConnector[I2C_COLORSENSORV2] == NULL)  {
          i2CConnector[I2C_COLORSENSORV2] = new Adafruit_TCS34725();
          ((Adafruit_TCS34725*)i2CConnector[I2C_COLORSENSORV2])->begin();
        }
        uint16_t rgbc[4];
        ((Adafruit_TCS34725*)i2CConnector[I2C_COLORSENSORV2])->getRawData(&rgbc[0], &rgbc[1], &rgbc[2], &rgbc[3]);
        sendShort( (double)(rgbc[pin]/rgbc[3]*256));
        break;
      }
#ifdef INCLUDE_JEULIN
    //--------------------
    // Finger HR Sensor
    //--------------------
    case SENSOR_HEARTRATE_AVAIL:
      {
        //int id = getI2Cid("fingerHR");
        if (i2CConnector[I2C_FINGERHEARTRATE] == NULL)  i2CConnector[I2C_FINGERHEARTRATE] = new Seeed_FingerHeartRate();
        Seeed_FingerHeartRate *tmp = (Seeed_FingerHeartRate*)i2CConnector[I2C_FINGERHEARTRATE];
        if ((tmp) && (tmp->isAvailable()))
        {
          sendByte( tmp->isAvailable()) ;
        } else {
          unsigned char error = 0;
          sendFloat(error);
        }
      }
      break;
    case SENSOR_HEARTRATE_READ:
      {
        //int id = getI2Cid("fingerHR");
        if (i2CConnector[I2C_FINGERHEARTRATE] == NULL)  i2CConnector[I2C_FINGERHEARTRATE] = new Seeed_FingerHeartRate();
        Seeed_FingerHeartRate *tmp = (Seeed_FingerHeartRate*)i2CConnector[I2C_FINGERHEARTRATE];
        if ((tmp) && (tmp->isAvailable()))
        {
          sendFloat( (float)tmp->read()) ;
        } else {
          float error = -1.0;
          sendFloat(error);
        }
      }
      break;
    //--------------------
    // CO2 gas Sensor
    //--------------------
    case SENSOR_CO2_AVAIL:
      {
        if (digitalConnector[pin] == NULL)  digitalConnector[pin] = new Seeed_GasSensor(pin, pin + 1);
        sendByte( ((Seeed_GasSensor *)digitalConnector[pin])->readSensor() );
        break;
      }
    case SENSOR_CO2_READ:
      {
        if (digitalConnector[pin] == NULL)  digitalConnector[pin] = new Seeed_GasSensor(pin, pin + 1);
        sendFloat( (float)((Seeed_GasSensor *)digitalConnector[pin])->litValeur( readBuffer(_PARAM1_) ) );
        break;
      }
    //--------------------
    case SENSOR_O2_READ:
      {
        pin = analogs[pin];
        pinMode(pin, INPUT);
        sendFloat(analogRead(pin));
        break;
      }
    //--------------------
    case SENSOR_DUST_READ:
      {
        if (digitalConnector[pin] == NULL)  digitalConnector[pin] = new Seeed_DustSensor(pin, pin + 1);
        sendFloat( (float)((Seeed_DustSensor *)digitalConnector[pin])->readConcentration() );
        break;
      }
    //--------------------
    // Color Sensor
    //--------------------
    case SENSOR_COLORRGB:
      {
        // Bug must be redeclared every time (?)
        if (i2CConnector[I2C_COLORSENSOR] == NULL)  {
          i2CConnector[I2C_COLORSENSOR] = new Seeed_ColorSensor();
          ((Seeed_ColorSensor*)i2CConnector[I2C_COLORSENSOR])->init(1);
        }
        sendShort( (double)((Seeed_ColorSensor*)i2CConnector[I2C_COLORSENSOR])->litCouleur(readBuffer(_PARAM1_)) );
        break;
      }

    //--------------------
    // Compass Sensor
    //--------------------
    case SENSOR_COMPASS3D:
      {
        //int id = getI2Cid("compass");
        if (i2CConnector[I2C_COMPASS] == NULL)  i2CConnector[I2C_COMPASS] = new Adafruit_HMC5883_Unified(12345);
        Adafruit_HMC5883_Unified *tmp = (Adafruit_HMC5883_Unified*)i2CConnector[I2C_COMPASS];
        if ((tmp) && (tmp->isAvailable()))
        {
          sendFloat( tmp->readAngle(readBuffer(_PARAM1_))) ;
        } else {
          float error = -36000.0;
          sendFloat(error);
        }
        break;
      }
    case ACCELEROMETER:
      {
        if (i2CConnector[I2C_ACCELEROMETER] == NULL)
        {
          i2CConnector[I2C_ACCELEROMETER] = new ADXL345();
          ((ADXL345*)i2CConnector[I2C_ACCELEROMETER])->powerOn();
        }
        int x,y,z;  
        ((ADXL345*)i2CConnector[I2C_ACCELEROMETER])->readXYZ(&x, &y, &z); //read the accelerometer values and store them in variables  x,y,z
        byte axis = readBuffer(6);
        if(axis == 0)
        {
          sendFloat((float)x);
        }
        else if(axis == 1)
        {
          sendFloat((float)y);
        }
        else
        {
          sendFloat((float)z);
        }
        break;
      }

    //--------------------
    // FINGER
    //--------------------
    case SENSOR_FINGERPRINT_DELETE:
      {
        if (digitalConnector[pin] == NULL)   digitalConnector[pin] = new Adafruit_Fingerprint(new SoftwareSerial(pin, pin + 1), true);
        sendByte ( ((Adafruit_Fingerprint*)digitalConnector[pin])->deleteAllFingerprint() );
        break;
      }
    case SENSOR_FINGERPRINT_ENROLLPASS1:
      {
        if (digitalConnector[pin] == NULL)   digitalConnector[pin] = new Adafruit_Fingerprint(new SoftwareSerial(pin, pin + 1), true);
        sendByte ( ((Adafruit_Fingerprint*)digitalConnector[pin])->enrollFingerPrintPass1(readBuffer(_PARAM1_)) );
        break;
      }
    case SENSOR_FINGERPRINT_ENROLLPASS2:
      {
        if (digitalConnector[pin] == NULL)   digitalConnector[pin] = new Adafruit_Fingerprint(new SoftwareSerial(pin, pin + 1), true);
        sendByte ( ((Adafruit_Fingerprint*)digitalConnector[pin])->enrollFingerPrintPass2(readBuffer(_PARAM1_)) );
        break;
      }
    case SENSOR_FINGERPRINT_READAVAILABLE:
      {
        if (digitalConnector[pin] == NULL)   digitalConnector[pin] = new Adafruit_Fingerprint(new SoftwareSerial(pin, pin + 1), true);
        //int test = (int)((Adafruit_Fingerprint*)digitalConnector[pin])->getFingerprintIDez();
        //sendShort ( test );
        if (((Adafruit_Fingerprint*)digitalConnector[pin])->getFingerprintIDez())
        {
          sendByte ( 0x1 );
        } else {
          sendByte ( 0x00 );
        }
        break;
      }
    case SENSOR_FINGERPRINT_READID:
      {
        if (digitalConnector[pin] == NULL)   digitalConnector[pin] = new Adafruit_Fingerprint(new SoftwareSerial(pin, pin + 1), true);
        int id;
        //for (int i=0; i<5; i++)
        {
          id = (int)((Adafruit_Fingerprint*)digitalConnector[pin])->getIDnumber2(); //  max id 167!!

        }
        sendByte ( id & 0xff );
        break;
      }
#endif //JEULIN
    case RF433_AVAILABLE:
      if (digitalConnector[pin] == NULL)
      {
        digitalConnector[pin] = new RCSwitch();
        ((RCSwitch*)digitalConnector[pin])->enableReceive(digitalPinToInterrupt(pin));
      }
      sendByte(((RCSwitch*)digitalConnector[pin])->available());
      break;
    case RF433_RECEIVE:
      if (digitalConnector[pin] == NULL)
      {
        digitalConnector[pin] = new RCSwitch();
        ((RCSwitch*)digitalConnector[pin])->enableReceive(digitalPinToInterrupt(pin));
      }
      sendShort(((RCSwitch*)digitalConnector[pin])->getReceivedValue());
      ((RCSwitch*)digitalConnector[pin])->resetAvailable();
      break;
    case ROBUNO_READ:
      sendByte(robuno.read(pin));
      break;
    case ROBUNO_LDR:
      sendShort((double)robuno.ldr_read(pin));
      break;

    case APPINVENTOR_RECEIVE:
      {
        int len = readBuffer(7);
        String prefix;
        for (byte i = 8; i < 8 + len; i++)
        {
          prefix += buffer[i];
        }
        String s = bt.receive(pin);
        int ret = 0;
        updateComVariable(s, prefix, ret);
        sendShort((double)ret);
        break;
      }
    case APPINVENTOR_AVAILABLE:
      {
        sendFloat(bt.dataAvailable(pin, pin + 1));
      }
      break;
  }
}
